'use strict';

var _ = require('lodash');

/**
 * Add a content item if none with the same name/key is found
 * @param  {string}     key     Key to match the name in entries
 * @param  {[object]}   entries Array of objects of the form {name: '...', content: '...'}
 * @param  {any}        value   Value for content of the created item
 * @return {object}     item    Existing item if one was found with the given key, the added item otherwise.
 */
module.exports.defaultValue = function(key, entries, value) {
    var item = _.find(entries, function(entry) {
        return entry.name === key;
    });
    if (item) {
        return item;
    }
    item = {
        name: key,
        content: value
    }
    entries.push(item);
    return item;
};
