# errorTracker

Uses Bugsnag under the hood

* Adds a collect() function to the original Bugsnag package, which initializes Bugsnag. All original Bugsnag methods remain available.
* Provides middleware-ready functions to be integrated into your project.

## Integration

To integrate Bugsnag into a project's backend, you need to apply the following steps.

### 1. Integrate middleware

#### Using Loopback
```
{
    "initial": {
        "coorpacademy-core/lib/errorTracker#track": {},
        ...
    },
    ...
    "final:after": {
        ...
        "coorpacademy-core/lib/errorTracker#handleError": {}, // <-- before the error handler
        ...
    }
}
```

#### Other
* Put the track middleware `errorTracker.middleware.track` as the first (or among the first) middleware
* Put the handleError middleware `errorTracker.middleware.handleError` right before the 5XX codes handle

### 2. Call errorTracker.collect()

Call errorTracker.collect() somewhere in your application. Try to call it as early as possible in your application in order to improve the crash coverage, as crashes that happen before errorTracker is initialized will not be collected.

```js
var errorTracker = require('coorpacademy-core').errorTracker;

// ...

errorTracker.collect(config, version);
```

#### errorTracker.collect(config[, version])

Initializes Bugsnag with the proper apiKey and options and eventual version.
config.options will be used as just like you would when initializing Bugsnag yourself (see the [docs](https://bugsnag.com/docs/notifiers/node#configuration)).

Required:
* server.apiKey

Default values:
* releaseStage: process.env.NODE_ENV
* version: process.env.npm_package_version

```js
var config = {
    "options": {
        "filters": ["password"],
        "notifyReleaseStages": ["production", "staging", "test", "development"]
    },
    "server": {
        "apiKey": "xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx" // Required
    }
};

var version = '1.0.0'; // Hardcoded
// or
var version = require('/path/to/package.json').version; // Reading package.json
// or
var version = process.env.npm_package_version; // When launched using `npm start/test` (default if available)
```
