'use strict';

var _ = require('lodash');
var bugsnag = require('bugsnag');
var Promise = require('bluebird');
var hasBeenInit = false;

bugsnag.collect = function(config, version) {
    if (hasBeenInit) {
        return;
    }
    if (!config || !config.server || !config.server.apiKey) {
        throw new Error('missing errorTracker server.apiKey');
    }

    // Add settings and any useful piece of information that we can send to Bugsnag
    var defaultSettings = {
        appVersion: version || process.env.npm_package_version,
        releaseStage: process.env.NODE_ENV
    };
    var settings = _.assign({}, config.options, defaultSettings);
    bugsnag.register(config.server.apiKey, settings);

    // Handle promises
    function notifyCrash(error) {
        console.error('Unhandled rejection: ' + (error && error.stack || error));
        bugsnag.notify(error);
    }

    // unhandledRejection === uncaughtException but for Promises
    process.on('unhandledRejection', notifyCrash);
    // The unhandledRejection event is being overriden by bluebird, so we override the default handler.
    Promise.onPossiblyUnhandledRejection(notifyCrash);

    hasBeenInit = true;
};

module.exports = bugsnag;
