'use strict';

var util = require('util');
var errorTracker = require('../errorTracker');
var morgan = require('morgan');

var logger = {};

// Simply pass everything to console.log|info
// (for some reason, I couldn't find out how to write tests for
// logger.log = console.log
// or
// logger.log = console.log.bind(console)
// )
['log', 'info'].forEach(function(type) {
    logger[type] = function() {
       console[type].apply(console, Array.apply(null, arguments));
   };
});

var bugsnagSeverityByType = {
    warn: 'warning',
    error: 'error'
};

/*
 * Will log a message using console.warn|error and send it formatted to Bugsnag.
 */
['warn', 'error'].forEach(function(type) {
    logger[type] = function() {
        if (arguments.length === 0) {
            return;
        }
        console[type].apply(console, Array.apply(null, arguments));

        errorTracker.notify(util.format.apply(util, arguments), {
            severity: bugsnagSeverityByType[type]
        });
    };
});

/*
 * Will log an error using console.error(error.stack || error) and send the error to Bugsnag.
 */
logger.errorStack = function(error) {
    if (!error) {
        return;
    }
    console.error(error.stack || error);

    errorTracker.notify(error, {
        severity: 'error'
    });
};

logger.wrap = function(error) {
    error.id = 'ERR' + new Date().getTime();
    error.errmsg = error.message;
    return error;
};

logger.expressLoggerMiddleware = function(options) {
    morgan.token('sessionID', function(req) {
        return req.sessionID;
    });

    morgan.token('login', function(req) {
        return req.user ? req.user.login : null;
    });

    morgan.token('env', function() {
      return process.env.NODE_ENV;
    });

    morgan.token('brand', function() {
      return process.env.NODE_BRAND;
    });

    morgan.token('buildIdentifier', function() {
      return process.env.BUILD_IDENTIFIER;
    });

    morgan.token('testIdentifier', function() {
      return process.env.TEST_IDENTIFIER;
    });

    morgan.format('coorpacademy', ':remote-addr [:date] ' +
        '":method :url HTTP/:http-version" :status :res[content-length] ":referrer"' +
        ' ":user-agent" service=:response-time login=:login sessionID=:sessionID :env' +
        ' :brand :buildIdentifier :testIdentifier');
    return morgan('coorpacademy', options);
};

module.exports = logger;
